// Copyright 2024 XMOS LIMITED.
// This Software is subject to the terms of the XMOS Public Licence: Version 1.

#include <xcore/hwtimer.h>
#include "asrc_task.h"
#include "app_build_config.h"

extern unsigned receive_asrc_input_samples_cb_default(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate);

#if (CONFIG == ADAT)
// Called from a different tile hence channel usage.
// It is invoked when an ISR caused by a channel tx is received at ASRC
ASRC_TASK_ISR_CALLBACK_ATTR
unsigned receive_asrc_input_samples(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate){
    return receive_asrc_input_samples_cb_default(c_digital_rx_to_asrc, asrc_io, new_input_rate);
}
#endif

#if (CONFIG == SPDIF)
extern unsigned spdif_receive_asrc_input_samples(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate);

// Called from a different tile (ASRC tile) hence channel usage.
ASRC_TASK_ISR_CALLBACK_ATTR
unsigned receive_asrc_input_samples(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate){
    return spdif_receive_asrc_input_samples(c_digital_rx_to_asrc, asrc_io, new_input_rate);
}
#endif

#if (CONFIG == I2S)
extern unsigned i2s_receive_asrc_input_samples(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate);

// Called from a different tile (ASRC tile) hence channel usage.
ASRC_TASK_ISR_CALLBACK_ATTR
unsigned receive_asrc_input_samples(chanend c_digital_rx_to_asrc, asrc_in_out_t *asrc_io, unsigned *new_input_rate){
    return receive_asrc_input_samples_cb_default(c_digital_rx_to_asrc, asrc_io, new_input_rate);
}
#endif

// Register the above function for ASRC task
void init_asrc_io_callback_wrapper(asrc_in_out_t *asrc_io){
    init_asrc_io_callback(asrc_io, receive_asrc_input_samples);
}
